from random import gauss as normal_rvs
from pyvmp.utils import discrete_rvs

from pyvmp.graph import ModelGraph
from pyvmp.inference_engine import InferenceEngine

from pyvmp.nodes.scalar import Gaussian, Gamma
from pyvmp.nodes.vector import Dirichlet, Discrete, ConditionalDiscrete
from pyvmp.nodes.mixture import MixtureDistribution


def main():
    data, labels = draw_markov_chain_sample()
    
    print "Fitting"
    
    # A container for the nodes in the model.
    model = ModelGraph()
    
    # Priors
    pi = Dirichlet({'alpha' : [1, 1, 1]})
    
    model.add_node(pi)
    
    A = [
         Dirichlet({'alpha' : [1, 1, 1]}),
         Dirichlet({'alpha' : [1, 1, 1]}),
         Dirichlet({'alpha' : [1, 1, 1]})
         ]
    model.add_node(A[0])
    model.add_node(A[1])
    model.add_node(A[2])
    
    component_priors = [
                         {'mean' : Gaussian({'mean' : 1, 'precision' : 10}), 'precision' : 1},
                         {'mean' : Gaussian({'mean' :-1, 'precision' : 10}), 'precision' : 1},
                         {'mean' : Gaussian({'mean' : 50, 'precision' : 10}), 'precision' : 1}
                        ]
    
    model.add_node(component_priors[0]['mean'])
#    model.add_node(component_priors[0]['precision'])
    model.add_node(component_priors[1]['mean'])
    model.add_node(component_priors[2]['mean'])
#    model.add_node(component_priors[1]['precision'])

    # Add first component
    hidden_states = [Discrete({'pi' : pi})]
    model.add_node(hidden_states[-1])
    
    parents = {}
    parents['index'] = hidden_states[-1]
    parents['components'] = component_priors
    
    visible_node = MixtureDistribution(Gaussian, parents, value=data[0])    
    model.add_node(visible_node)
    
    # Add the remaining components
    for i in range(1, len(data)):       
        hidden_states.append(ConditionalDiscrete({'X' : hidden_states[-1], 'A' : A}))
        model.add_node(hidden_states[-1])        
        
        # Normal components
        parents = {}
        parents['index'] = hidden_states[-1]
        parents['components'] = component_priors
        
        visible_node = MixtureDistribution(Gaussian, parents, value=data[i])    
        model.add_node(visible_node)

    # Helps convergence if we update the Dirichlet and indicator node first.
#    mix_weight_priors.update()
    for node in model.nodes:
        if isinstance(node, (Discrete, ConditionalDiscrete)):
            node.update()            
    
    # Create the inference engine and run.
    ie = InferenceEngine(model, verbose=True)
    ie.run()

    print pi.e_x
    
    print
    
    for a in A:
        print a.e_x
    
    print
    
    for comp in component_priors:
        print comp['mean'].e_x #comp['precision'].e_x
    
    print
    
    for i in range(100):
        print data[i], labels[i], hidden_states[i].e_x[labels[i]]
    

def draw_markov_chain_sample():
    pi = [0.6, 0.3, 0.1]
    
    T = [
         [0.8, 0.1, 0.1],
         [0.4, 0.5, 0.1],
         [0.2, 0.1, 0.7]
         ]
    
    E = [10, -30, 100]
    
    s = [discrete_rvs(pi)]
    e = [normal_rvs(E[s[0]], 10)] 
    
    for i in range(100000):
        state = discrete_rvs(T[s[-1]])
        observed_value = normal_rvs(E[state], 10)
        
        s.append(state)
        e.append(observed_value)
        
    return e, s

if __name__ == "__main__":
    main()
