from __future__ import division

import random

from math import sqrt

from pyvmp.graph import ModelGraph
from pyvmp.nodes.scalar import Gamma, Gaussian
from pyvmp.nodes.vector import Dirichlet, Discrete
from pyvmp.nodes.mixture import MixtureDistribution

from pyvmp.inference_engine import InferenceEngine

'''
Example of bayesian inference with a Gaussian mixture distribution, with a normal prior on the means and gamma prior on
the precisions.
'''
mu = [100, -100, 0]

cluster_sizes = [100000, 1000, 100]

gamma = 10

# A container for the nodes in the model.
model = ModelGraph()

# Create mixture parents
components_parents = []

for m in mu:
    precision = Gamma({'a' : 1, 'b': 1})    
    mean = Gaussian({'mean' : random.gauss(m, 10), 'precision' : 2})
    
    model.add_node(precision)
    model.add_node(mean)
    
    components_parents.append({'mean' : mean, 'precision' : precision})

pi = Dirichlet({'alpha' : [1e-6, 1e-6, 1e-6]})
model.add_node(pi)

# Add data points
for c, size in enumerate(cluster_sizes):
    for i in range(size):
        index = Discrete({'pi' : pi})
        model.add_node(index)
        
        parents = {'components' : components_parents, 'index' : index}       
        value = random.gauss(mu[c], sqrt(1 / gamma))        
        node = MixtureDistribution(Gaussian, parents, value=value)
        model.add_node(node)
        
pi.update()
for node in model.nodes:
    if isinstance(node, Discrete):
        node.update()    

# Create the inference engine and run.
ie = InferenceEngine(model, verbose=True)
ie.run()

#i = 1
#for node in model.nodes:
#    if isinstance(node, MixtureDistribution):
#        mw = node.mix_weights
#        
#        print i, mw.index(max(mw)), node.value, mw
#        
#        i += 1

# Print out the expected value of the mean and precision distributions. These should be close to mu and gamma.
print "#" * 50
print pi.e_x

for single_comp_parents in components_parents:
    print single_comp_parents['mean'].e_x, single_comp_parents['precision'].e_x,
    print single_comp_parents['mean'].parents['mean'].e_x 
