from pyvmp.inference_engine import InferenceEngine
from pyvmp.nodes.vector import Dirichlet, Discrete, Multinomial
from pyvmp.nodes.mixture import MixtureDistribution
from pyvmp.graph import ModelGraph
from pyvmp.utils import multinomial_rvs

def multinomial_mixture_model(mu, distribution_priors, cluster_sizes):
    # A container for the nodes in the model.
    model = ModelGraph()
    
    parents = load_parents(distribution_priors, model)
    
    # Add data points
    for c, size in enumerate(cluster_sizes):
        for i in range(size):
            parents['index'] = Discrete({'pi' : parents['pi']})
            model.add_node(parents['index'])
            
            value = multinomial_rvs(10, mu[c])
            
            node = MixtureDistribution(Multinomial, parents, value=value)
        
            model.add_node(node)

    # Helps convergence if we update the Dirichlet and indicator node first.
    parents['pi'].update()
    for node in model.nodes:
        if isinstance(node, Discrete):
            node.update()            
    
    # Create the inference engine and run.
    ie = InferenceEngine(model)
    converged = ie.run(max_iters=1000)
    
    if not converged:
        print parents['pi'].e_x
        
        for single_comp_parents in parents['components']:
            print single_comp_parents['pi'].e_x
        
def load_parents(distribution_priors, model):
    components_parents = []
    
    # Component priors.
    for alpha in distribution_priors:
        multinomial_prior = Dirichlet({'alpha' : alpha})
        model.add_node(multinomial_prior)
        
        components_parents.append({'pi' : multinomial_prior})
    
    # Mix-weight prior
    mix_weight_prior = Dirichlet({'alpha' : [1e-6, 1e-6, 1e-6]})
    model.add_node(mix_weight_prior)
    
    parents = {}
    parents['components'] = components_parents
    parents['pi'] = mix_weight_prior
    
    return parents

if __name__ == "__main__":
    mu = [
          [0.99, 0.01],
          [0.6, 0.4],
          [0.01, 0.99]
          ]

    distribution_priors = [
                           [10, 1],
                           [5, 5],
                           [1, 10]
                           ]
    
    cluster_sizes = [1000, 100, 0]
    
    for i in range(100):
        multinomial_mixture_model(mu, distribution_priors, cluster_sizes)

