'''
Created on 2011-11-17

@author: Andrew Roth
'''
import random

class InferenceEngine(object):
    def __init__(self, graph, verbose=False, debug=False):
        self.nodes = graph.nodes
        
        self.verbose = verbose
        
        self._set_nodes_to_update()
        
        if debug:
            self.update = self.debug_update
        else:
            self.update = self.normal_update

    def _set_nodes_to_update(self):
        self._nodes_to_update = []
        
        for node in self.nodes:
            if node.observed:
                continue
            
            self._nodes_to_update.append(node)

        random.shuffle(self._nodes_to_update)
        
    def run(self, max_iters=100, tolerance=1e-6):
        bounds = [float('-inf')]
        
        converged = False
        
        iters = 0
        
        while not converged:
            self.update()

            bounds.append(self.get_bound())

            convergence_check = self.check_convergence(bounds, tolerance)
            
            if convergence_check == 1:
                converged = True
                
                break
            
            elif convergence_check == -1:
                converged = False
                
                break
            
            iters += 1
            
            if iters >= max_iters:
                break
        
        return converged
    
    def check_convergence(self, bounds, tolerance):
        if self.verbose:
            print bounds[-1]
        
        diff = (bounds[-1] - bounds[-2])
        
        if abs(diff) < tolerance:
            print "Converged"
            
            return 1
        
        elif diff < 0:            
            print "Bound decreased", diff
            
            return -1
        
        else:
            return 0

    def normal_update(self):
        for node in self._nodes_to_update:           
            node.update()
    
    def debug_update(self):
        for node in self._nodes_to_update:
            old_bound = self.get_bound()
            
            node.update()
            
            new_bound = self.get_bound()
            
            if new_bound - old_bound < -1e-3:
                print new_bound - old_bound, node
                node.update()
    
    def get_bound(self):
        bound = 0
        
        for node in self.nodes:
            bound += node.get_bound()
        
        return bound
