'''
Created on 2011-11-22

@author: Andrew Roth
'''
class ContianerNode(object):
    def __init__(self, node_class, parents, size, values=None):
        self.node_class = node_class
        self.size = size
        
        if values is not None:
            self.observed = True
        
        self.nodes = []
        
        if len(parents) == 1:
            self._init_single_parents(parents[0], values)
        elif len(parents) == size:
            self._init_multiple_parents(parents, values)
        else:
            raise Exception("Either one set of parents, or a number of parents matching the size must be passed.")
        
    def _init_single_parents(self, parents, values):
        for i in range(self.size):
            if self.observed:
                new_node = self.node_class(parents, values[i])
            else:
                new_node = self.node_class(parents)

            self.nodes.append(new_node)
    
    def _init_multiple_parents(self, parents, values):
        for i in range(self.size):
            if self.observed:
                new_node = self.node_class(parents[i], value=values[i])                
            else:
                new_node = self.node_class(parents[i])

            self.nodes.append(new_node)
    
    def update(self):
        for node in self.nodes:
            node.update()
            
    def get_bound(self):
        bound = 0
        
        for node in self.nodes:
            bound += node.get_bound()
        
        return bound
    
    def update_prior(self):
        for node in self.nodes:
            node.update_prior()
    
    def get_expected_log_p(self):
        expected_log_p = 0
        
        for node in self.nodes:
            expected_log_p += node.get_expected_log_p()
        
        return expected_log_p
    
    @property
    def parent_child_list(self):
        parent_child_list = []
        
        for node in self.nodes:
            parent_child_list.extend(node.parent_child_list)
        
        return parent_child_list
            
            
        
